import '../common/name.dart';
import '../common/named_api_resource.dart';
import '../common/named_api_resource_list.dart';
import '../../network/rest.dart';

///Egg Groups are categories which determine which Pokémon are able to interbreed.
///Pokémon may belong to either one or two Egg Groups.
///Check out [Bulbapedia](http://bulbapedia.bulbagarden.net/wiki/Egg_Group) for greater detail.
class EggGroup {
  int id;
  String name;
  List<Name> names;
  List<NamedApiResource> pokemonSpecies;

  EggGroup.fromJson(Map<String, dynamic> json)
      : id = json['id'],
        name = json['name'],
        names = Name.fromList(json['names'] ?? []),
        pokemonSpecies =
            NamedApiResource.fromList(json['pokemon_species'] ?? []);

  static List<EggGroup> fromList(List<dynamic> list) =>
      list.map((e) => EggGroup.fromJson(e ?? {})).toList();

  static Future<EggGroup> get(String query) async =>
      EggGroup.fromJson(await RestConnection.get<EggGroup>(query));

  static Future<NamedApiResourceList> find(
          [int limit = 0, int skip = 0]) async =>
      NamedApiResourceList.fromJson(
          await RestConnection.find<EggGroup>(limit, skip));

  @override
  toString() =>
      '$runtimeType {id: $id, name: $name, names: $names, pokemonSpecies: $pokemonSpecies}';
}
